import { graph } from './decorators/Graph';
import { inject } from './decorators/inject/Inject';
import { injectable } from './decorators/injectable/Injectable';
import { lateInject } from './decorators/lateInject/LateInject';
import { lifecycleBound } from './decorators/LifecycleBound';
import { provides } from './decorators/provides/Provides';
import { singleton } from './decorators/singleton/Singleton';
import _Obsidian from './Obsidian';

export * from './types';

export { graph } from './decorators/Graph';
export { singleton } from './decorators/singleton/Singleton';
export { ObjectGraph } from './graph/ObjectGraph';
export { Graph as IGraph } from './graph/Graph';
export { provides } from './decorators/provides/Provides';
export { injectable } from './decorators/injectable/Injectable';
export { inject } from './decorators/inject/Inject';
export { lateInject } from './decorators/lateInject/LateInject';
export { lifecycleBound } from './decorators/LifecycleBound';
export { GraphMiddleware } from './graph/registry/GraphMiddleware';
export { GraphResolveChain as ResolveChain } from './graph/registry/GraphResolveChain';

/**
 * @deprecated Uppercase decorators are deprecated in favor of lowercase decorators to align with common naming conventions. Please use `@graph` instead.
 */
export const Graph = graph;
/**
 * @deprecated Uppercase decorators are deprecated in favor of lowercase decorators to align with common naming convention. Please use `@singleton` instead.
 */
export const Singleton = singleton;
/**
 * @deprecated Uppercase decorators are deprecated in favor of lowercase decorators to align with common naming convention. Please use `@provides` instead.
 */
export const Provides = provides;
/**
 * @deprecated Uppercase decorators are deprecated in favor of lowercase decorators to align with common naming convention. Please use `@injectable` instead.
 */
export const Injectable = injectable;
/**
 * @deprecated Uppercase decorators are deprecated in favor of lowercase decorators to align with common naming convention. Please use `@inject` instead.
 */
export const Inject = inject;
/**
 * @deprecated Uppercase decorators are deprecated in favor of lowercase decorators to align with common naming convention. Please use `@lateInject` instead.
 */
export const LateInject = lateInject;
/**
 * @deprecated Uppercase decorators are deprecated in favor of lowercase decorators to align with common naming convention. Please use `@lifecycleBound` instead.
 */
export const LifecycleBound = lifecycleBound;


export const Obsidian = new _Obsidian();

export { injectComponent } from './injectors/components/InjectComponent';
export { injectHook, injectHookWithArguments } from './injectors/hooks/InjectHook';

export { useObserver } from './observable/useObserver';
export { useObservers } from './observable/useObservers';
export { Observable } from './observable/Observable';
export { MediatorObservable } from './observable/mediator/MediatorObservable';
export { OnNext, Unsubscribe } from './observable/types';

export { Model } from './model/Model';

export { testKit } from '../testkit';
export { mockModel, mockGraphs } from '../testkit/index';
